import typer

from .melting import chat_prompts
from .search import search
from .plugins import plugins
from .private_workspaces import private_workspaces


def main():
    app = typer.Typer()
    app.add_typer(search.app, name="search")
    app.add_typer(chat_prompts.app, name="chat-prompts")
    app.add_typer(plugins.app, name="plugins")
    app()


if __name__ == "__main__":
    main()



import datetime
from pathlib import Path
from typing import NotRequired, TypedDict

from melting_schemas.meta import Creator


class IndexingSettings(TypedDict):
    model: str
    parsers: NotRequired[dict[str, str]]


class PrivateWorkspaceCreationRequest(TypedDict):
    name: str
    indexing_settings: IndexingSettings


class PrivateWorkspaceCreationResponse(PrivateWorkspaceCreationRequest):
    created_at: datetime.datetime
    created_by: Creator
    workspace_id: str


class PrivateWorkspace(TypedDict):
    id: str
    created_by: Creator
    name: str
    indexing_settings: IndexingSettings


class PrivateFile(TypedDict):
    created_by: Creator
    workspace_id: str
    filepath: Path | str
    filename: str
    filetype: str
    content: NotRequired[str]
    content_hash: NotRequired[str]
    content_type: NotRequired[str]
    media_type: NotRequired[str]
    size_bytes: int


class OperationExecution(TypedDict):
    created_at: datetime.datetime
    message: NotRequired[str]
    status: str


class IndexingOperation(TypedDict):
    type: str
    executions: NotRequired[list[OperationExecution]]


class PrivateWorkspaceIndexing(TypedDict):
    id: str
    workspace_id: str
    operations: NotRequired[list[IndexingOperation]]



from .services.chat_prompts import TemplatingClient
from .services.fcall_completions import FCallCompletionsClient
from .services.text_encodings import TextEncodingClient
from .services.chat_completions import CompletionClient
from .services.tcall_completions import TCallCompletionsClient


class MFClient:
    chat_prompts = TemplatingClient
    completion = CompletionClient
    encoding = TextEncodingClient
    fcall_completion = FCallCompletionsClient
    tcall_completion = TCallCompletionsClient



from datetime import datetime
from typing import Literal, Optional, TypedDict

from melting_schemas.completion.chat import (
    ChatMLMessage,
    StreamTimings,
    Timings,
    TokenUsage,
)
from melting_schemas.historian import Templating
from melting_schemas.meta import Creator


class ChatCompletionResponse(TypedDict):
    _id: str
    created_at: datetime
    created_by: Creator
    finish_reason: Literal["stop", "length"]
    messages: list[ChatMLMessage]
    output: ChatMLMessage
    settings: dict
    templating: Optional[Templating]
    timings: Timings | StreamTimings
    usage: TokenUsage



import json

import httpx
import requests
from http_error_schemas.factory import get_error_class
from rich import print

from .exceptions import TeiaSdkError


def pjson(obj):
    """Prettify JSON."""
    return json.dumps(obj, indent=4, sort_keys=True)


def ppjson(obj):
    """Pretty print JSON."""
    print(pjson(obj))


def handle_erros(response: requests.Response | httpx.Response):
    try:
        response.raise_for_status()
    except (requests.HTTPError, httpx.HTTPStatusError) as e:
        exc = get_error_class(response.status_code)
        obj = response.json()
        ppjson(exc(**obj))
        raise TeiaSdkError(obj, e.request, e.response) from e
    except (requests.RequestException, httpx.RequestError) as e:
        print(e.request)
        raise TeiaSdkError(response.json()) from e



import pytest

from teia_sdk import PluginClient
from teia_sdk.plugins.schemas import ChatMLMessage, FCallModelSettings
from teia_sdk.utils import ppjson


class TestPluginsClient:
    @pytest.fixture
    def client(self):
        return PluginClient()

    def test_aveilable(self, client: PluginClient):
        assert len(client.available_plugins().keys()) > 3

    def test_select_and_run_plugin(self, client: PluginClient):
        response = client.select_and_run_plugin(
            model_settings=FCallModelSettings(model="gpt-3.5-turbo-1106"),
            messages=[ChatMLMessage(role="user", content="Who is Simion Petrov?")],
            plugin_names=["internal_search"],
        )
        assert "Simion Petrov" in response["plugin_infos"][0]["params"]["query"]
        assert response["plugin_infos"][0]["name"] == "internal_search"

    def test_select_plugin(self, client: PluginClient):
        response = client.run_selector(
            model_settings=FCallModelSettings(model="gpt-3.5-turbo-1106"),
            messages=[ChatMLMessage(role="user", content="Who is Simion Petrov?")],
            plugin_names=["internal_search"],
        )
        ppjson(response)
        assert "Simion Petrov" in response["arguments"]["query"]
        assert response["plugin"] == "internal_search"

    def test_run_plugin(self, client: PluginClient):
        plugin_calls = [
            {
                "plugin": "internal_search",
                "method": "internal_search",
                "arguments": {"query": "Who is Monica Pastor?"},
            }
        ]
        response = client.run_plugins(plugin_calls=plugin_calls)
        assert "Monica Pastor" in response["plugin_infos"][0]["params"]["query"]
        assert response["plugin_infos"][0]["name"] == "internal_search"
        assert response["plugin_infos"][0]["name"] == "internal_search"



from pathlib import Path
from typing import Annotated, Optional
import json

import typer
from rich import print
from typer import Option

from .client import SearchClient
from ..utils import ppjson

app = typer.Typer()


@app.command()
def find(
    query: Annotated[str, Option("-q", "--query")],
    model_name: Annotated[str, Option("-m", "--model_name")],
    model_type: Annotated[str, Option("-t", "--model_type")],
    kb_name: Annotated[str, Option("-kb", "--kb_name")],
    threshold: Annotated[float, Option("-th", "--threshold")] = 0.5,
    top_k: Annotated[int, Option("-tk", "--top_k")] = 10,
):
    """Find similar documents in a knowledge base."""
    body = {
        "query": query,
        "model_name": model_name,
        "model_type": model_type,
        "search_settings": [
            {
                "kb_name": kb_name,
                "threshold": threshold,
                "top_k": top_k,
            }
        ],
    }
    response = SearchClient.search(body=body)
    print(ppjson(response))



import os


try:
    TEIA_API_KEY = os.environ["TEIA_API_KEY"]
    MELT_API_URL = os.getenv("MELT_API_URL", "https://meltingface.teialabs.com.br/api")
except KeyError:
    m = "[red]MissingEnvironmentVariables[/red]: "
    m += "[yellow]'TEIA_API_KEY'[/yellow] cannot be empty."
    print(m)
    exit(1)




import json
import os
import httpx

from .schemas import SearchRequest, SearchResponse

try:
    TEIA_API_KEY = os.environ["TEIA_API_KEY"]
    SEARCH_API_URL = os.getenv("SEARCH_API_URL", "https://plugins.athena.teialabs.com.br")
except KeyError:
    m = "[red]MissingEnvironmentVariables[/red]: "
    m += "[yellow]'TEIA_API_KEY'[/yellow] cannot be empty."
    print(m)
    exit(1)


class SearchClient:
    relativepath = "/search/"  # TODO remove slash

    @classmethod
    def get_headers(cls) -> dict[str, str]:
        obj = {
            "Authorization": f"Bearer {TEIA_API_KEY}",
        }
        return obj

    @classmethod
    def search(cls, body: SearchRequest) -> SearchResponse:
        response = httpx.post(
            url=f"{SEARCH_API_URL}{cls.relativepath}",
            headers=cls.get_headers(),
            json=body,
        )
        return response.json()



from typing import Any, Optional

import httpx
from melting_schemas.templating import ChatPromptTemplate
from melting_schemas.templating.prompt import ChatPrompt, GeneratedFields

from ...utils import handle_erros
from .. import MELT_API_URL, TEIA_API_KEY


class TemplatingClient:
    relative_path = "/templating/chat-prompts"

    @classmethod
    def get_headers(cls) -> dict[str, str]:
        obj = {
            "Authorization": f"Bearer {TEIA_API_KEY}",
        }
        return obj

    @classmethod
    def read_one(
        cls, identifier: Optional[str] = None, name: Optional[str] = None
    ) -> ChatPrompt:
        if identifier is not None:
            res = httpx.get(
                f"{MELT_API_URL}{cls.relative_path}/{identifier}",
                headers=cls.get_headers(),
            )
        elif name is not None:
            res = httpx.get(
                f"{MELT_API_URL}{cls.relative_path}",
                headers=cls.get_headers(),
                params={"name": name},
            )
        else:
            raise ValueError("Must provide either 'identifier' or 'name'.")
        handle_erros(res)
        return res.json()

    @classmethod
    def read_many(
        cls,
        limit: int,
        skip: int,
        name: Optional[str] = None,
        model: Optional[str] = None,
    ) -> list[ChatPrompt]:
        params = {"name": name, "settings.model": model, "$limit": limit, "$skip": skip}
        params = {k: v for k, v in params.items() if v is not None}
        res = httpx.get(
            f"{MELT_API_URL}{cls.relative_path}",
            headers=cls.get_headers(),
            params=params,
        )
        handle_erros(res)
        return res.json()

    @classmethod
    def post(cls, body: ChatPromptTemplate) -> GeneratedFields:
        res = httpx.post(
            f"{MELT_API_URL}{cls.relative_path}",
            headers=cls.get_headers(),
            json=body,
        )
        handle_erros(res)
        return res.json()

    @classmethod
    def delete(cls, name: str) -> dict[str, Any]:
        res = httpx.delete(
            f"{MELT_API_URL}{cls.relative_path}/{name}",
            headers=cls.get_headers(),
        )
        try:
            res.raise_for_status()
            return {"status_code": res.status_code}
        except httpx.HTTPError:
            body = res.json()
            return {"status_code": res.status_code, **body}



import json
import os
from typing import Any, Iterator, Optional

import httpx
import requests
from melting_schemas.historian.chat_completions import (
    ChatCompletionCreationResponse,
    ChatCompletionRequest,
    HybridChatCompletionRequest,
    RawChatCompletionRequest,
    StreamedChatCompletionCreationResponse,
)

from ...exceptions import TeiaSdkError
from ...utils import handle_erros
from .. import MELT_API_URL, TEIA_API_KEY
from ..schemas import ChatCompletionResponse


class CompletionClient:
    relative_path = "/text-generation/chat-completions"
    timeout = 60

    @classmethod
    def get_headers(cls) -> dict[str, str]:
        obj = {
            "Authorization": f"Bearer {TEIA_API_KEY}",
        }
        return obj

    @classmethod
    def create_one(
        cls,
        body: ChatCompletionRequest
        | RawChatCompletionRequest
        | HybridChatCompletionRequest,
        user_email: Optional[str] = None,
    ) -> ChatCompletionCreationResponse:
        if not isinstance(body, dict):
            body = body.dict(exclude_none=True)

        headers = cls.get_headers()
        if user_email:
            headers["X-User-Email"] = user_email

        res = httpx.post(
            f"{MELT_API_URL}{cls.relative_path}/create",
            timeout=cls.timeout,
            headers=headers,
            json=body,
        )

        return res.json()

    @classmethod
    def read_one(cls, identifier: str) -> ChatCompletionResponse:
        res = httpx.get(
            f"{MELT_API_URL}{cls.relative_path}/{identifier}",
            headers=cls.get_headers(),
        )
        handle_erros(res)
        return res.json()

    @classmethod
    def stream_one(
        cls,
        body: ChatCompletionRequest
        | RawChatCompletionRequest
        | HybridChatCompletionRequest,
        count_tokens: bool = False,
        user_email: Optional[str] = None,
    ) -> tuple[str, Iterator[StreamedChatCompletionCreationResponse]]:
        if not isinstance(body, dict):
            body = body.dict(exclude_none=True)

        headers = cls.get_headers()
        if count_tokens:
            headers["X-Count-Tokens"] = "true"
        if user_email:
            headers["X-User-Email"] = user_email
        # TODO: use httpx stream instead of requests
        res = requests.post(
            f"{MELT_API_URL}{cls.relative_path}/stream",
            headers=headers,
            json=body,
            stream=True,
        )
        try:
            res.raise_for_status()
        except requests.HTTPError as e:
            raise TeiaSdkError(res.json()) from e
        identifier = res.headers["Content-Location"].split("/")[-1]
        return identifier, map(json.loads, res.iter_lines())



import logging

import dotenv

dotenv.load_dotenv()
logging.basicConfig(level=logging.CRITICAL)  # here to prevent log.WARN clutter

from .search.client import SearchClient
from .melting.client import MFClient
from .plugins.client import PluginClient
from .private_workspaces.client import PrivateWorkspaceClient

__all__ = ["SearchClient", "MFClient", "PluginClient", "PrivateWorkspaceClient"]



from typing import Any, Optional, TypedDict
from melting_schemas.completion.fcall import ChatMLMessage, FCallModelSettings


class PluginInfo(TypedDict):
    name: str
    method: str
    params: dict[str, Any]
    response: str | dict[str, Any]
    error: str
    response_time: int


class PluginResponse(TypedDict):
    selector_completion: str
    plugin_selection_id: str
    plugin_execution_id: str
    plugins_infos: Optional[list[PluginInfo] | None]
    error: str


class SelectPlugin(TypedDict):
    messages: list[ChatMLMessage]
    plugin_names: list[str]
    model_settings: FCallModelSettings
    plugin_extra_arguments: dict[str, dict[str, str]]


class PluginUsage(TypedDict):
    plugin: str
    method: str
    arguments: dict[str, Any]


class PluginCall(TypedDict):
    plugin: str
    method: str
    arguments: dict[str, Any]


class GetPluginSelection(TypedDict):
    plugin_selector: SelectPlugin
    plugin_usage: list[PluginCall]
    completion_id: str
    error: str


class GetPluginExecution(TypedDict):
    plugin_calls: list[PluginCall]
    plugin_infos: list[PluginInfo]
    error: str



from pathlib import Path
from typing import Annotated, Optional
import json

import typer
from rich import print
from typer import Option

from .client import MFClient
from ..utils import ppjson

app = typer.Typer()


@app.command()
def put(
    name: Annotated[str, Option("-n", "--name")],
    file: Annotated[
        Path,
        Option(
            "-f", "--file", exists=True, file_okay=True, readable=True, allow_dash=True
        ),
    ],
):
    print("put", repr(name), repr(file))


@app.command()
def read_one(
    identifier: Annotated[Optional[str], Option("--id", metavar="ID")] = None,
    name: Annotated[Optional[str], Option("-n", "--name")] = None,
):
    if identifier is None and name is None:
        typer.echo("Must provide either 'identifier' or 'name'.")
        raise typer.Abort()
    ppjson(MFClient.chat_prompts.read_one(identifier=identifier, name=name))


@app.command()
def read_many(
    name: Annotated[Optional[str], Option("-n", "--name")] = None,
    model: Annotated[Optional[str], Option("-m", "--model")] = None,
    limit: Annotated[int, Option("-l", "--limit")] = 10,
    skip: Annotated[int, Option("-s", "--skip")] = 0,
):
    ppjson(
        MFClient.chat_prompts.read_many(name=name, model=model, limit=limit, skip=skip)
    )


@app.command()
def post(
    file: Annotated[
        Path,
        Option(
            "-f", "--file", exists=True, file_okay=True, readable=True, allow_dash=True
        ),
    ],
):
    ppjson(MFClient.chat_prompts.post(json.load(file.open())))


@app.command()
def delete(
    name: Annotated[str, Option("-n", "--name")],
):
    if name is None:
        typer.echo("Must provide 'name'.")
        raise typer.Abort()
    else:
        ppjson(MFClient.chat_prompts.delete(name))



from ..exceptions import TeiaSdkError


class ErrorToGetPluginInfo(TeiaSdkError):
    """Error to get plugin info"""


class ErrorToGetPluginResponse(TeiaSdkError):
    """Error to get plugin info"""


class ErrorPluginAPISelectAndRun(TeiaSdkError):
    """API was not able to select and run plugins"""


class PluginSelectionNotFound(TeiaSdkError):
    """API was not able to find the plugin selection object"""


class PluginExecutionNotFound(TeiaSdkError):
    """API was not able to find the plugin execution object"""


class ErrorGetPluginSelection(TeiaSdkError):
    """API was not able to select and run plugins"""


class ErrorGetPluginExecution(TeiaSdkError):
    """API was not able to select and run plugins"""



from typing import Optional, TypedDict


class SearchSettings(TypedDict):
    kb_name: str
    threshold: float
    top_k: int


class SearchRequest(TypedDict):
    query: str
    model_name: str
    model_type: str
    search_settings: list[SearchSettings]


class SearchResult(TypedDict):
    content: str
    data_type: str
    file_id: str
    kb_name: str
    query: str
    url: str
    instance_id: str
    relevance: int


class SearchResponse(TypedDict):
    results: list[SearchResult]



import typer

from .client import PrivateWorkspaceClient
from ..utils import ppjson

app = typer.Typer()


# TODO
@app.command()
def get_private_workspaces():
    response = PrivateWorkspaceClient.get_private_workspaces()
    ppjson(response)



import logging
import os
from pathlib import Path
from typing import Optional
from fastapi import UploadFile

from starlette import status as http_status
import httpx

from .exceptions import (
    PrivateWorkspaceFileNotFound,
    PrivateWorkspaceNotFound,
)
from .schemas import (
    PrivateFile,
    PrivateWorkspace,
    PrivateWorkspaceCreationRequest,
    PrivateWorkspaceCreationResponse,
    PrivateWorkspaceIndexing,
)

from ..utils import handle_erros

try:
    TEIA_API_KEY = os.environ["TEIA_API_KEY"]
    DATASOURCES_API_URL = os.getenv(
        "DATASOURCES_API_URL", "https://datasources.teialabs.com.br"
    )
except KeyError:
    m = "[red]MissingEnvironmentVariables[/red]: "
    m += "[yellow]'TEIA_API_KEY'[/yellow] cannot be empty."
    print(m)
    exit(1)


class PrivateWorkspaceClient:
    relative_path = "/api/workspaces"

    @classmethod
    def get_headers(cls) -> dict[str, str]:
        obj = {
            "Authorization": f"Bearer {TEIA_API_KEY}",
        }
        return obj

    @classmethod
    def get_headers_with_user_email(cls, user_email: Optional[str] = None) -> dict[str, str]:
        headers = cls.get_headers()
        if user_email is not None:
            headers["X-User-Email"] = user_email
        return headers

    @classmethod
    def upload_many_files(
        cls,
        workspace_id: str,
        file_paths: Optional[list[Path]] = None,
        files: Optional[list[UploadFile]] = None,
        user_email: Optional[str] = None,
    ):
        """
        Uploads files to be processed.
        """
        if files is None and file_paths is None:
            raise ValueError("Either 'files' or 'file_paths' must be provided.")

        if files is None:
            files = []
            for p in file_paths:
                files.append(("files", open(p, "rb")))

        headers = cls.get_headers_with_user_email(user_email)
        res = httpx.put(
            f"{DATASOURCES_API_URL}{cls.relative_path}/{workspace_id}/files",
            headers=headers,
            files=files,
        )
        return res.json()

    @classmethod
    def create_private_workspace(
        cls,
        workspace: PrivateWorkspaceCreationRequest,
        user_email: Optional[str] = None,
    ) -> PrivateWorkspaceCreationResponse:
        """
        Create a private workspace.
        """
        headers = cls.get_headers_with_user_email(user_email)
        res = httpx.post(
            f"{DATASOURCES_API_URL}{cls.relative_path}/",
            headers=headers,
            json=workspace,
        )
        handle_erros(res)
        return res.json()

    @classmethod
    def get_many_private_workspaces(
        cls,
        sort: Optional[str] = None,
        order: Optional[str] = None,
        limit: Optional[int] = None,
        offset: Optional[int] = None,
        workspace_id: Optional[str] = None,
        name: Optional[str] = None,
        user_email: Optional[str] = None,
    ) -> list[PrivateWorkspace]:
        """
        Retrieve a list of private workspaces.
        """
        params = {
            "sort": sort,
            "order": order,
            "limit": limit,
            "offset": offset,
            "workspace_id": workspace_id,
            "name": name,
        }
        params = {k: v for k, v in params.items() if v is not None}
        headers = cls.get_headers_with_user_email(user_email)
        res = httpx.get(
            f"{DATASOURCES_API_URL}{cls.relative_path}/",
            headers=headers,
            params=params,
        )
        handle_erros(res)
        return res.json()

    @classmethod
    def get_private_workspace(
        cls, workspace_id: str, user_email: Optional[str] = None
    ) -> PrivateWorkspace:
        """
        Get a private workspace.
        """
        headers = cls.get_headers_with_user_email(user_email)
        res = httpx.get(
            f"{DATASOURCES_API_URL}{cls.relative_path}/{workspace_id}/",
            headers=headers,
        )
        if res.status_code == http_status.HTTP_404_NOT_FOUND:
            raise PrivateWorkspaceNotFound(f"Workspace '{workspace_id}' not found.")
        handle_erros(res)
        return res.json()

    @classmethod
    def list_file(
        cls,
        workspace_id: str,
        file_path: str | Path,
        return_file_content: Optional[bool] = None,
        user_email: Optional[str] = None,
    ) -> PrivateFile:
        """
        Show a specific file in a given private workspace.
        """
        headers = cls.get_headers_with_user_email(user_email)
        res = httpx.get(
            f"{DATASOURCES_API_URL}{cls.relative_path}/{workspace_id}/files/{file_path}",
            headers=headers,
            params={"return_content": return_file_content},
        )
        if res.status_code == http_status.HTTP_404_NOT_FOUND:
            raise PrivateWorkspaceFileNotFound(f"File '{file_path}' not found.")

        handle_erros(res)
        return res.json()

    @classmethod
    def list_many_files(
        cls,
        workspace_id: str,
        file_path: Optional[str] = None,
        file_name: Optional[str] = None,
        file_type: Optional[str] = None,
        return_file_content: Optional[bool] = None,
        limit: Optional[int] = None,
        offset: Optional[int] = None,
        sort: Optional[str] = None,
        order: Optional[str] = None,
        user_email: Optional[str] = None,
    ) -> list[PrivateFile]:
        """
        Shows files in a given private workspace.
        """
        params = {
            "file_path": file_path,
            "file_name": file_name,
            "file_type": file_type,
            "return_file_content": return_file_content,
            "limit": limit,
            "offset": offset,
            "sort": sort,
            "order": order,
        }
        params = {k: v for k, v in params.items() if v is not None}
        headers = cls.get_headers_with_user_email(user_email)
        res = httpx.get(
            f"{DATASOURCES_API_URL}{cls.relative_path}/{workspace_id}/files/",
            headers=headers,
            params=params,
        )
        handle_erros(res)
        return res.json()

    @classmethod
    def upload_file(
        cls, workspace_id: str, file_path: Path, file: Optional[UploadFile] = None,
        user_email: Optional[str] = None
    ) -> PrivateFile:
        """
        Uploads a file to be processed.
        """
        if file is None:
            file = open(file_path, "rb")
        file = {"file": file}

        headers = cls.get_headers_with_user_email(user_email)
        res = httpx.put(
            f"{DATASOURCES_API_URL}{cls.relative_path}/{workspace_id}/files/{file_path}",
            headers=headers,
            files=file,
        )
        handle_erros(res)
        return res.json()

    @classmethod
    def delete_file(cls, workspace_id: str, file_path: str | Path, user_email: Optional[str] = None):
        """
        Delete a file from a private workspace.
        """
        headers = cls.get_headers_with_user_email(user_email)
        res = httpx.delete(
            f"{DATASOURCES_API_URL}{cls.relative_path}/{workspace_id}/files/{file_path}",
            headers=headers,
        )
        handle_erros(res)
        return res

    @classmethod
    def create_indexing(cls, workspace_id: str, user_email: Optional[str] = None) -> PrivateWorkspaceIndexing:
        """
        Uploads a file to be processed.
        """
        headers = cls.get_headers_with_user_email(user_email)
        res = httpx.post(
            f"{DATASOURCES_API_URL}{cls.relative_path}/{workspace_id}/indexings/",
            headers=headers,
        )
        handle_erros(res)
        return res.json()

    @classmethod
    def get_indexing(
        cls,
        workspace_id: str,
        indexing_id: str,
        user_email: Optional[str] = None,
    ) -> PrivateWorkspaceIndexing:
        """
        Get information from an indexing process.
        """
        headers = cls.get_headers_with_user_email(user_email)
        res = httpx.get(
            f"{DATASOURCES_API_URL}{cls.relative_path}/{workspace_id}/indexings/{indexing_id}",
            headers=headers,
        )
        handle_erros(res)
        return res.json()

    @classmethod
    def get_many_indexings(
        cls,
        workspace_id: str,
        limit: Optional[int] = None,
        offset: Optional[int] = None,
        sort: Optional[str] = None,
        order: Optional[str] = None,
        user_email: Optional[str] = None,
    ) -> list[PrivateWorkspaceIndexing]:
        """
        Get information from all indexing processes in a workspace.
        """
        params = {
            "limit": limit,
            "offset": offset,
            "sort": sort,
            "order": order,
        }
        params = {k: v for k, v in params.items() if v is not None}
        headers = cls.get_headers_with_user_email(user_email)
        res = httpx.get(
            f"{DATASOURCES_API_URL}{cls.relative_path}/{workspace_id}/indexings/",
            headers=headers,
            params=params,
        )
        handle_erros(res)
        return res.json()

    @classmethod
    def delete_private_workspace(cls, workspace_id: str, user_email: Optional[str] = None):
        """
        Delete workspace and all resources related to it.
        """
        headers = cls.get_headers_with_user_email(user_email)
        res = httpx.delete(
            f"{DATASOURCES_API_URL}{cls.relative_path}/{workspace_id}",
            headers=headers,
        )
        handle_erros(res)
        return res

    @classmethod
    def search_on_workspace(
        cls,
        workspace_id: str,
        query: str,
        num_results: Optional[int] = None,
        schemaless: Optional[bool] = None,
        user_email: Optional[str] = None,
    ) -> list[dict]:
        """
        Perform an embedding search in a private workspace.
        """
        params = {
            "query": query,
            "num_results": num_results,
            "schemaless": schemaless,
        }
        params = {k: v for k, v in params.items() if v is not None}
        headers = cls.get_headers_with_user_email(user_email)
        res = httpx.post(
            f"{DATASOURCES_API_URL}{cls.relative_path}/{workspace_id}/search/",
            headers=headers,
            params=params,
        )
        handle_erros(res)
        return res.json()



import httpx
from typing import Optional
from melting_schemas.completion.tcall import (
    RawTCallRequest,
    TCallCompletionCreationResponse,
)

from .. import TEIA_API_KEY, MELT_API_URL
from ...utils import handle_erros


class TCallCompletionsClient:
    relative_path = "/text-generation/tcall-completions"
    client = httpx.Client(timeout=60)

    @classmethod
    def get_headers(cls) -> dict[str, str]:
        obj = {
            "Authorization": f"Bearer {TEIA_API_KEY}",
        }
        return obj

    @classmethod
    def create_one(
        cls, body: RawTCallRequest, user_email: Optional[str] = None
    ) -> TCallCompletionCreationResponse:
        headers = cls.get_headers()
        if user_email:
            headers["X-User-Email"] = user_email

        res = cls.client.post(
            f"{MELT_API_URL}{cls.relative_path}/create",
            headers=headers,
            json=body.dict(),
        )
        handle_erros(res)
        return res.json()



import logging
import os
from typing import Any, Optional

import httpx
from melting_schemas.completion.fcall import ChatMLMessage, FCallModelSettings

from starlette import status as http_status

from . import exceptions
from .schemas import (
    GetPluginExecution,
    GetPluginSelection,
    PluginInfo,
    PluginResponse,
    PluginUsage,
    SelectPlugin,
)

logger = logging.getLogger(__name__)

try:
    TEIA_API_KEY = os.environ["TEIA_API_KEY"]
    PLUGINS_API_URL = os.getenv(
        "PLUGINS_API_URL", "https://plugins.athena.teialabs.com.br"
    )
except KeyError:
    m = "[red]MissingEnvironmentVariables[/red]: "
    m += "[yellow]'TEIA_API_KEY'[/yellow] cannot be empty."
    print(m)
    exit(1)


class PluginClient:
    client = httpx.Client(timeout=120)

    @classmethod
    def get_headers(cls) -> dict[str, str]:
        obj = {
            "Authorization": f"Bearer {TEIA_API_KEY}",
        }
        return obj

    @classmethod
    def available_plugins(cls) -> dict[str, dict[str, Any]]:
        res = httpx.get(
            f"{PLUGINS_API_URL}/available",
            headers=cls.get_headers(),
        )
        return res.json()

    @classmethod
    def select_and_run_plugin(
        cls,
        messages: list[ChatMLMessage],
        plugin_names: list[str],
        model_settings: FCallModelSettings,
        plugin_extra_args: Optional[dict[str, dict[str, str]]] = None,
        user_email: Optional[str] = None,
        schemaless: bool = True,
    ) -> PluginResponse:

        if not plugin_names:
            return PluginResponse(
                selector_completion="",
                plugin_infos=[],
                error=f"No plugins in plugin_names",
            )

        if plugin_extra_args is None:
            plugin_extra_args = {}
        for plugin in plugin_extra_args:
            plugin_extra_args[plugin]["schemaless"] = schemaless

        sp = SelectPlugin(
            messages=messages,
            plugin_names=plugin_names,
            model_settings=model_settings,
            plugin_extra_arguments=plugin_extra_args,
        )

        sel_run_url = f"{PLUGINS_API_URL}/select-and-run-plugin"
        headers = cls.get_headers()
        if user_email:
            headers["X-User-Email"] = user_email

        logger.debug(f"Requesting {sel_run_url}. Args: {sp}. Headers: {headers}.")
        try:
            plugins_data = cls.client.post(sel_run_url, json=sp, headers=headers)
            logger.debug(
                f"Request returned: {plugins_data}, {plugins_data.status_code}."
            )
        except httpx.ReadTimeout as ex:
            raise exceptions.ErrorPluginAPISelectAndRun(
                f"Request to {sel_run_url} timed out\nError: {ex}. "
            )
        except Exception as ex:
            raise exceptions.ErrorPluginAPISelectAndRun(
                f"Request to {sel_run_url} did not work\nError: {ex}. "
            )

        if plugins_data.status_code != http_status.HTTP_200_OK:
            raise exceptions.ErrorPluginAPISelectAndRun(
                f"Request: {sel_run_url}\njson: {sp}\nError: {plugins_data.status_code}: {plugins_data.text}. "
            )

        try:
            plugins_data = plugins_data.json()
        except AttributeError:
            raise exceptions.ErrorToGetPluginResponse(
                f"Tried to convert response to json. Response: {plugins_data}. "
            )

        plugins_data["plugin_infos"] = [
            PluginInfo(**p) for p in plugins_data["plugin_infos"]
        ]
        plugins_data = PluginResponse(**plugins_data)

        return plugins_data

    @classmethod
    def run_selector(
        cls,
        messages: list[ChatMLMessage],
        plugin_names: list[str],
        model_settings: FCallModelSettings,
    ) -> PluginUsage:
        sp = SelectPlugin(
            messages=messages,
            plugin_names=plugin_names,
            model_settings=model_settings,
        )

        plugins_selected = cls.client.post(
            f"{PLUGINS_API_URL}/select-plugin",
            json=sp,  # errado
            headers=cls.get_headers(),
        )

        plugins_selected.raise_for_status()

        plugin_calls = PluginUsage(**plugins_selected.json()["plugin_usage"][0])
        return plugin_calls

    @classmethod
    def run_plugins(cls, plugin_calls: list[PluginUsage]):
        plugin_data = httpx.post(
            f"{PLUGINS_API_URL}/run-plugin",
            json=plugin_calls,
            headers=cls.get_headers(),
            timeout=120,
        )

        plugin_data.raise_for_status()

        plugin_data = plugin_data.json()
        plugin_data = PluginResponse(**plugin_data)

        return plugin_data

    @classmethod
    def read_plugin_execution(cls, id: str, user_email: str) -> GetPluginExecution:
        headers = cls.get_headers()
        headers["X-User-Email"] = user_email

        res = httpx.get(
            f"{PLUGINS_API_URL}/run-plugin/{id}",
            headers=headers,
        )

        if res.status_code == http_status.HTTP_404_NOT_FOUND:
            raise exceptions.PluginExecutionNotFound(
                f"PluginExecution not found for {id} and email {user_email}. API returned: {res.json()}",
            )
        elif res.status_code != http_status.HTTP_200_OK:
            raise exceptions.ErrorGetPluginExecution(
                f"Error retrieving PluginExecution for {id} and email {user_email}. API returned: {res}"
            )

        try:
            res = res.json()
            res = GetPluginExecution(**res)
        except Exception as ex:
            raise exceptions.ErrorGetPluginExecution(
                f"Error while converting GET plugin exection response {res} to data schema: {ex}"
            )

        return res

    @classmethod
    def read_plugin_selection(cls, id: str, user_email: str) -> GetPluginSelection:
        headers = cls.get_headers()
        headers["X-User-Email"] = user_email

        res = httpx.get(
            f"{PLUGINS_API_URL}/select-plugin/{id}",
            headers=headers,
        )
        if res.status_code == http_status.HTTP_404_NOT_FOUND:
            raise exceptions.PluginSelectionNotFound(
                f"PluginSelection not found for {id} and email {user_email}: {res.json()}",
            )
        elif res.status_code != http_status.HTTP_200_OK:
            raise exceptions.ErrorGetPluginSelection(
                f"Error retrieving PluginSelection for {id} and email {user_email}: {res}"
            )

        try:
            res = res.json()
            res = GetPluginSelection(**res)
        except Exception as ex:
            raise exceptions.ErrorGetPluginSelection(
                f"Error while converting GET plugin selection response {res} to data schema: {ex}"
            )

        return res



import httpx
from melting_schemas.encoding.text_encoding import RawTextEncoding, TextEncodingResponse

from ...utils import handle_erros
from .. import MELT_API_URL, TEIA_API_KEY


class TextEncodingClient:
    relative_path = "/text-encodings"

    @classmethod
    def get_headers(cls) -> dict[str, str]:
        obj = {
            "Authorization": f"Bearer {TEIA_API_KEY}",
        }
        return obj

    @classmethod
    def encode(cls, body: RawTextEncoding) -> TextEncodingResponse:
        res = httpx.post(
            f"{MELT_API_URL}{cls.relative_path}",
            timeout=15,
            headers=cls.get_headers(),
            json=body.dict(),
        )
        handle_erros(res)
        return res.json()



from pathlib import Path
from typing import Annotated, Optional
import json

import typer
from rich import print
from typer import Option

from .client import PluginClient
from ..utils import ppjson

app = typer.Typer()


@app.command()
def available():
    response = PluginClient.available_plugins()
    ppjson(response)


@app.command()
def select():
    pass


@app.command()
def run():
    pass


@app.command()
def select_and_run():
    pass
